#!/usr/bin/python3

import psycopg2, subprocess, os, sys, random
import numpy as np
from random import randint 
from datetime import datetime, timedelta

NPART = 8   # Total course matter is divided in to 8 parts, to ensure uniformity
AN = 6		# Number of A,B and C type questions required
BN = 5
CN = 5

QTEXT = 0	# indices for the rows from the database
QATEXT = 1
QAUTHOR = 2
QPART = 3
QTYPE = 4
QSN = 5
QRELATIONS = 6
QMARK = 7
QSTAT = 8
QFIGNAME = 9
QFIG = 10
QUSED = 11    # added by this program
QPART2 = 12   # added by this program

SINGLEQL = 12	# length of the list after adding the 'used' flag

host = 'localhost'
ourDB = 'kuqbank'

def AQgen(AQ,BQ,CQ, dt):
	print('Python: Entering AQGen..')
	#--------Generate Latex document, using questions & answers from AQ,BQ and CQ -----
	tex = '\\documentclass[12pt, a4paper]{exam}\n \\usepackage{graphicx}\n \\usepackage{amsmath}\n \\usepackage{amssymb}\n'
	tex += '\\usepackage[left=0.8in, top=0.7in, bottom=0.7in]{geometry}\n'

	if language.lower() == 'malayalam':
		tex += '\\usepackage{fontspec}\n\\usepackage{polyglossia}\n'
		tex += '\\setmainfont[Script=Malayalam]{Rachana}'			
	elif language.lower() == 'hindi':
		tex += '\\usepackage{fontspec}\n\\usepackage{polyglossia}\n'
		tex += '\\setmainfont[Script=Devanagari]{Lohit Devanagari}\n'		

	tex += '\\begin{document} \\noindent\n'

	tex += '%s \\hfill %s\n\\newline '%(qpCode,dt)
	
	tex += '\\begin{center} \\large\n \\textbf {%s Semester %s %s Examination\n'%(semester, degree, subject)
	tex += '%s %s\n'%(examMonth, examYear)
	tex += '(%s Admission onwards)\\\\\n'%syllabusYear
	tex += '%s (%s)}\n\\end{center}\n'%(courseCode,courseName)
	
	tex += '\\noindent\n' 
	tex += 'Time : %s  \hfill Maximum Marks : %2d\n\\vspace{0.1in}\n\n'%(duration,MAXMARK)
	tex += '\\hrule\n\\vspace{0.1in}\n'

	if qpModel == 'M48':
		s = 'Answer all six questions'
	else:
		s = 'Answer any %d questions'%AQN
	tex += '{\\small \\bfseries  part A %s. (Each carries %d marks) }'%(s,AMARK)

	tex += '\\begin{questions} \\pointsdroppedatright\n'

	for q in AQ:
		tex += '\\question[%d]P%d(%d): %s\n\n\\textbf{Answer:}\n\n%s\n\\droppoints\n'\
			%(q[QMARK],q[QPART],q[QSN],q[QTEXT].strip(),q[QATEXT].strip())
		tex += '\\vspace{0.05in}\n'

	tex += '{\\small \\bfseries  part B (Answer any 3 questions. Each carries %d marks) }'%BMARK

	for q in BQ:
		tex += '\\question[%d]P%d(%d): %s\n\n\\textbf{Answer:}\n\n%s\\droppoints\n'\
			%(q[QMARK],q[QPART],q[QSN],q[QTEXT].strip(),q[QATEXT].strip())
		tex += '\\vspace{0.05in}\n'
	tex += '{\\small \\bfseries  part C (Answer any 3 questions. Each carries %d marks) }'%CMARK

	for q in CQ:
		if len(q) == SINGLEQL:
			tex += '\\question[%d]P%d(%d): %s\n\n\\textbf{Answer:}\n\n%s\\droppoints\n'\
				%(q[QMARK],q[QPART],q[QSN],q[QTEXT].strip(),q[QATEXT].strip())
			tex += '\\vspace{0.05in}\n'
		else:
			q2 = q[QPART2]
			tex += '\\question\n'
			tex += '\\begin{parts} \n'
			tex += '\\part[%d]P%d(%d): %s\n\n\\textbf{Answer:}\n\n%s\\droppoints\n'\
				%(q[QMARK],q[QPART],q[QSN],q[QTEXT].strip(),q[QATEXT].strip())
			tex += '\\part[%d]P%d(%d): %s\n\n\\textbf{Answer:}\n\n%s\\droppoints\n'\
				%(q2[QMARK],q2[QPART],q2[QSN],q2[QTEXT].strip(),q2[QATEXT].strip())
			tex += '\\end{parts}'	
			tex += '\\vspace{0.05in}\n'

	tex += '\\end{questions}\n'
	tex += '\\end{document}'

	fn = courseCode + '_ANS'
	open(fn + '.tex', 'w').write(tex)
	print('Python: Written Answer .tex file ',fn)
	
def makeTexQP():
	if courseCode == None:
		print('No Course Selected')
		return	
	try:
		conn = psycopg2.connect(database=ourDB, host=host, user=courseCode.lower(), password = passwd)
		cursor = conn.cursor()	
		cursor.execute("SELECT * FROM %s"%courseCode)	
		results = cursor.fetchall()
	except:
		print('Connection to Database "%s" failed. Check password.'%ourDB)
		sys.exit('Database connection failed')
		
	# Count the number of questions in each section, type wise
	naq = np.zeros(NPART, dtype=int)
	nbq = np.zeros(NPART, dtype=int)
	ncq = np.zeros(NPART, dtype=int)
	for row in results:
		for k in range(NPART):		# k goes from 0 to NPART-1. Add 1 while comparing
			if row[QTYPE] == 'A' and row[QPART] == k+1:
				naq[k] += 1
			if row[QTYPE] == 'B' and row[QPART] == k+1:
				nbq[k] += 1
			if row[QTYPE] == 'C' and row[QPART] == k+1:
				ncq[k] += 1
	
	print(naq, nbq,ncq)
	# Make 3 array of objects, size NPART, to transfer the SQL query results
	Atypes = np.empty(NPART,object)
	Btypes = np.empty(NPART,object)
	Ctypes = np.empty(NPART,object)

	for k in range(NPART):		# Each element holds N arrays, for questions from NPART sections
		Atypes[k] = np.empty(naq[k], object)
		Btypes[k] = np.empty(nbq[k], object)
		Ctypes[k] = np.empty(ncq[k], object)

	nrec = len(results)			# results is from SQL query
	for k in range(nrec):		# Copy Query results to three 2D Array of objects
		row = list(results[k])
		#print(row)
		if row[QSTAT] == 'inactive':
			row.append(True)				# mark it as used, also set when selected for paper
			#print(row)
		else:
			row.append(False)				
		
		if row[QTYPE] == 'A':
			sec = row[QPART]
			qn = row[QSN]
			Atypes[sec-1][qn-1] = row	# array indices are from 0 to 7 (not 1 to NPART)
		if row[QTYPE] == 'B':
			sec = row[QPART]
			qn = row[QSN]
			Btypes[sec-1][qn-1] = row
		if row[QTYPE] == 'C':
			sec = row[QPART]
			qn = row[QSN]
			Ctypes[sec-1][qn-1] = row
		if row[QFIGNAME] != None:
			f = open(row[QFIGNAME],'wb')
			f.write(row[QFIG])
			f.close()	
	#print('Fetched Questions')


	# Prepare for selecting questions for a QP
	def markRelations(part,relation): # Mark the questions inside relation as used
		if relation == '':
			return
		rels = relation.split(',')
		for r in rels:
			r = r.strip()
			rtype = r[0]
			rval = r[1:]
			rqn = eval(r[1:])-1
			if rtype == 'C':
				Ctypes[part][rqn][QUSED] = True
			if rtype == 'A':
				Atypes[part][rqn][QUSED] = True

	# Select the Parts to pick A,B and C type questions.
	ic = []
	for k in range(0,NPART,2):
		ic.append(randint(0,1)+k)
	while 1:
		n = randint(0,NPART-1)
		if n not in ic:
			ic.append(n)
			break

	ib = []
	for p in range(NPART):
		if p not in ic:
			ib.append(p)
	random.shuffle(ib)
	
	ia = ic.copy() 			# Five A types  from the same sections
	ia.append(ib[-1])		# Add third section of B to A	
	ib = (ib*2)[:-1]		# repeat the first two sections of B type 
	
	ia.sort()
	ib.sort()
	ic.sort()
	#print (ia, ib, ic)
	
	#Lists to store the selected questions
	AQ = []
	BQ = []
	CQ = []

	for k in range(CN):					# Select Ctype first
		part = ic[k]					# 5 pre-selected sections are stored in ic
		random.shuffle(Ctypes[part])
		for q in Ctypes[part]:
			if q[QMARK] >= 5 and q[QUSED] == False:
				q[QUSED] = True
				CQ.append(q)
				markRelations(part, q[QRELATIONS])
				if	q[QMARK] != CMARK:				# get one more and append 
					mr2 = CMARK - q[QMARK]
					random.shuffle(Ctypes[part])
					for q in Ctypes[part]:
						if q[QMARK] == mr2 and q[QUSED] == False:
							q[QUSED] = True
							CQ[-1].append(q)		#append to this C type question
							markRelations(part, q[QRELATIONS])
							break
				break
				
	for k in range(BN):			# Select Ctype first
		part = ib[k]			# 5 pre-selected sections of B
		random.shuffle(Btypes[part])
		
		for q in Btypes[part]:
			if q[QUSED] == False:
				q[QUSED] = True
				BQ.append(q)
				markRelations(part, q[QRELATIONS])
				break
	
	for k in range(AN):				# Select A type
		part = ia[k]				# Six pre-selected sections
		random.shuffle(Atypes[part])
		for q in Atypes[part]:
			if q[QUSED] == False:
				q[QUSED] = True
				AQ.append(q)
				markRelations(part, q[QRELATIONS])
				break

	#Verify the results.
	if len(AQ) != AN:
		print('Could get only %d number of A type questions. Requires %d'%(len(AQ),AN))
		sys.exit(1)
	if len(BQ) != BN:
		print('Could get only %d number of B type questions. Requires %d'%(len(BQ),BN))
		sys.exit(1)
	if len(CQ) != CN:
		print('Could get only %d number of C type questions. Requires %d'%(len(CQ),CN))
		sys.exit(1)
	
	for q1 in CQ:
		if q1[QMARK] != CMARK  and len(q1) == SINGLEQL:
			print('C type part1 %d marks. Could not find second part'%(q1[QMARK]))
			sys.exit(1)
					 	
	now = datetime.now() + timedelta(hours = 5, minutes=30)
	dt = now.strftime("%d/%m/%Y %H:%M:%S")
	AQgen(AQ, BQ, CQ, dt)

	print('Python: Answer Done. Making QP..')
	#--------Generate Latex document, using data from AQ,BQ and CQ -----
	tex = '\\documentclass[12pt, a4paper]{exam}\n \\usepackage{graphicx}\n \\usepackage{amsmath}\n \\usepackage{amssymb}\n'
	tex += '\\usepackage[left=0.8in, top=0.7in, bottom=0.7in]{geometry}\n'

	if language.lower() == 'malayalam':
		tex += '\\usepackage{fontspec}\n\\usepackage{polyglossia}\n'
		tex += '\\setmainfont[Script=Malayalam]{Rachana}'			
	elif language.lower() == 'hindi':
		tex += '\\usepackage{fontspec}\n\\usepackage{polyglossia}\n'
		tex += '\\setmainfont[Script=Devanagari]{Lohit Devanagari}\n'		

	tex += '\\begin{document}\n'
	
	tex += '\\begin{center}\n'
	tex += 'Reg No:.................. \\hfill \\small {%s}\n\n'%(qpCode)
	tex += 'Name :.................... \\hfill  \\small {%s}\n\n'%('')	
	
	tex += '\\large\n \\textbf {%s Semester %s %s Examination\n'%(semester, degree, subject)
	tex += '\\\\%s %s\n'%(examMonth, examYear)
	courseYear = courseCode[1:5]
	tex += '(%s Admission onwards)\\\\\n'%courseYear # syllabusYear
	tex += '%s (%s)}\n\n'%(courseCode[5:],courseName)
	tex += '(%s)\n\n'%qpHeader
	tex += '\\vspace{0.1in}\n\\end{center}\n '
	tex += '\\hrule\n\\vspace{0.1in}\n'
	
	tex += '\\noindent\n' 
	tex += 'Time : 3 hours  \hfill Maximum Marks : %2d\n\\vspace{0.1in}\n\n'%MAXMARK
	
	tex += '{\\noindent \\bfseries  Part A (Answer any 5 questions. Each carries %d marks) }'%AMARK

	tex += '\\begin{questions} \\pointsdroppedatright\n'

	for q in AQ:
		tex += '\\question[%d] %s \\droppoints\n'%(q[QMARK],q[QTEXT].strip())
		tex += '\\vspace{0.05in}\n'

	tex += '{\\noindent \\small \\bfseries  Part B (Answer any 3 questions. Each carries %d marks) }'%BMARK
	for q in BQ:
		tex += '\\question[%d] %s \\droppoints\n'%(q[QMARK],q[QTEXT].strip())
		tex += '\\vspace{0.05in}\n'
	
	tex += '{\\noindent\n\\small \\bfseries Part C (Answer any 3 questions. Each carries %d marks) }'%CMARK
	for q in CQ:
		if len(q) == SINGLEQL:
			tex += '\\question[%d] %s \\droppoints\n'%(q[QMARK],q[QTEXT].strip())
			tex += '\\vspace{0.05in}\n'
		else:
			q2 = q[QPART2]
			tex += '\\question\n'
			tex += '\\begin{parts} \n'
			tex += '\\part[%d] %s \\droppoints'%(q[QMARK], q[QTEXT].strip())
			tex += '\\part[%d] %s \\droppoints'%(q2[QMARK], q2[QTEXT].strip())
			tex += '\\end{parts}'	
			tex += '\\vspace{0.05in}\n'

	tex += '\\end{questions}\n'
	tex += '\\end{document}'

	fn = courseCode
	open(fn + '.tex', 'w').write(tex)
	print('Python: Written Question .tex file ',fn)

#---- main program -----

print ('Opening INFO ...')
ss = open('info.txt').read().split('\n')
print('Done')
#os.unlink('info.txt')
for row in ss:
	sss = row.split(':')
	print(sss)
	if sss[0] == 'examYear':
		examYear = sss[1]
	elif sss[0] == 'examMonth':
		examMonth = sss[1]
	elif sss[0] == 'courseCode':
		courseCode = sss[1]
	elif sss[0] == 'passwd':
		passwd = sss[1]
	elif sss[0] == 'qpCode':
		qpCode = sss[1]
	elif sss[0] == 'header':
		qpHeader = sss[1]

print(courseCode, passwd, examYear, examMonth, qpCode)

conn = psycopg2.connect(database=ourDB, host=host, user='dummy', password="dum123")
cursor = conn.cursor()
cursor.execute('select * from coursedetails;')
res = cursor.fetchall()
found = False
for row in res:
	#print (row[0], row[1])
	if row[0] == courseCode:
		found = True
		courseName = row[1]
		subject = row[2]
		degree = row[3]
		syllabusYear = row[4]
		semester = row[5]
		language = row[6]
		qpModel = row[7]
		duration = row[8]

if found == False:
	print('Course Code not found')
	sys.exit('Error')
		
if qpModel == 'M40':
	AMARK = 2
	BMARK = 4
	CMARK = 6
	AQN = 5			# answer any five 
	MAXMARK = 40

if qpModel == 'M32':
	AMARK = 1
	BMARK = 3
	CMARK = 6
	AQN = 5			# answer any five 
	MAXMARK = 32

if qpModel == 'M60':
	AMARK = 3
	BMARK = 5
	CMARK = 10
	AQN = 5			# answer any five 
	MAXMARK = 60

if qpModel == 'M80A':
	AMARK = 4
	BMARK = 7
	CMARK = 13
	AQN = 5			# answer any five 
	MAXMARK = 80

if qpModel == 'M80B':
	AMARK = 4
	BMARK = 8
	CMARK = 12
	AQN = 5			# answer any five 
	MAXMARK = 80
	
if qpModel == 'M48':
	AMARK = 1
	BMARK = 5
	CMARK = 9
	AQN = 6		# answer all 6 from A 
	MAXMARK = 48
	
print('%s %s %s %s %s %s %s %s'%(courseName, subject, degree, syllabusYear, semester, language,qpModel,duration))
print('Calling makeTexQP..')
makeTexQP()
print ('Done')
